using BPA.SAAS.Manage.Application.DataBase.Dtos.Batching;
using BPA.SAAS.Manage.Core.Base;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Interface
{
    public interface IBatchingService
    {
        Task<PageUtil> GetBatchingList(BatchingListQuery dto);
        Task<bool> AddBatching(BatchingInfoDto dto);
        Task<bool> UpdateBatching(BatchingInfoDto dto);
        Task<bool> BatchDelBatching(List<string> Ids);
        Task<Dictionary<string, object>> GetBatchingList();
        Task<List<BatchingList>> GetBatchingSelectList();
        Task<bool> UpdateBatchingStatus(BatchingStatusDto dto);
        Task<List<dynamic>> GetBatchingTypeList();
        Task<bool> AddBatchingType(BatchingTypeDto dto);
        Task<bool> BatchTypeDel(string id);
        Task<List<dynamic>> GetBatchingUintList();
        Task<bool> AddBatchingUint(BatchingUintDto dto);
        Task<bool> BatchingUintDel(string id);
        Task<List<BatchingView>> GetByTypeBatching();
        Task<List<dynamic>> GetBatchingTypeList_alm();
        Task<List<dynamic>> GetBatchingUintList_alm();
    }
}
